// $Id: CaptureConsole.java,v 1.7 2001/05/28 21:21:22 pcharles Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.client;

import java.util.Collection;
import java.util.HashSet;
import net.ultrametrics.console.TestConsole;
import net.sourceforge.jpcap.net.RawPacket;
import net.sourceforge.jpcap.capture.PacketCapture;
import net.sourceforge.jpcap.capture.CaptureStatistics;
import net.sourceforge.jpcap.capture.CapturePacketException;
import net.sourceforge.jpcap.capture.InvalidFilterException;
import net.sourceforge.jpcap.capture.CaptureDeviceOpenException;
import net.sourceforge.jpcap.capture.CaptureConfigurationException;


/**
 * Text console for controlling packet capture.
 *
 * @author Patrick Charles and Jonas Lehmann
 * @version $Revision: 1.7 $
 * @lastModifiedBy $Author: pcharles $
 * @lastModifiedAt $Date: 2001/05/28 21:21:22 $
 */
public class CaptureConsole extends TestConsole
{
  /**
   * Construct a new capture console.
   * @param captureTool the capture tool service handle.
   * @param captureSystem the packet capture system.
   */
  public CaptureConsole(CaptureTool captureTool, 
                        PacketCapture captureSystem) {
    this.captureTool = captureTool;
    this.captureSystem = captureSystem;
  }

  /**
   * Command processor. Handles commands specific to packet capture.
   * Invokes parent to handle commands inherited from other consoles.
   * @param command the command to execute.
   * @param args optional and required arguments to the provided command.
   */
  public String doCommand(String command, String [] args) {
    String result = "";
    if(command.toLowerCase().startsWith("device"))
      return command_device(args);
    if(command.toLowerCase().startsWith("snaplen"))
      return command_snaplen(args);
    if(command.toLowerCase().startsWith("timeout"))
      return command_timeout(args);
    if(command.toLowerCase().startsWith("filter"))
      return command_filter(args);
    if(command.toLowerCase().startsWith("status"))
      return command_status();
    if(command.toLowerCase().startsWith("capture"))
      return command_capture(args);
    if(command.toLowerCase().startsWith("mode"))
      return command_mode(args);
    if(command.toLowerCase().startsWith("history"))
      return command_history(args);
    if(command.toLowerCase().startsWith("clear"))
      return command_clear(args);
    if(command.toLowerCase().startsWith("ui"))
      return command_ui(args);

    result = super.doCommand(command, args);

    return result;
  }

  /**
   * Set the device name.
   */
  public String command_device(String [] args) {
    if(args.length < 1) 
      return "usage: device device_name";

    captureTool.setDevice(args[0]);
    return "device set to '" + args[0] + "'";
  }

  /**
   * Set the capture data length.
   */
  public String command_snaplen(String [] args) {
    if(args.length < 1) 
      return "usage: snaplen length";

    snaplen = Integer.parseInt(args[0]);
    return "snaplen set to " + snaplen;
  }

  /**
   * Set the capture timeout in ms.
   */
  public String command_timeout(String [] args) {
    if(args.length < 1) 
      return "usage: timeout length";

    timeout = Integer.parseInt(args[0]);
    return "timeout set to " + timeout;
  }

  /**
   * Set the filter expression.
   */
  public String command_filter(String [] args) {
    if(args.length < 1) 
      return "usage: filter filter_expression";

    filter = args[0];
    for(int i=1; i<args.length; i++)
      filter = filter + " " + args[i];

    return "filter set to '" + filter + "'";
  }

  /**
   * Set the capture mode.
   */
  public String command_mode(String [] args) {
    if(args.length < 1) 
      return "usage: mode_name\n  where mode_name is one of: \n" + 
        "\traw - display raw packet data during capture\n" + 
        "\tobject - display packet objects during capture\n" + 
        "\tverbose - display both raw packet data and network objects\n" +
        "\tterse - don't display any packet data during capture\n";

    String mode = args[0];
    if(!captureTool.isModeValid(mode))
      return "unrecognized mode '" + mode + "'";

    captureTool.setMode(mode);
    return "packet capture display mode toggled to '" + mode + "'";
  }

  /**
   * Clear the history buffer.
   */
  public String command_clear(String [] args) {
    captureTool.clearHistory();

    return "history cleared";
  }

  /**
   * Clear the history buffer.
   */
  public String command_ui(String [] args) {
    if(captureTool.activateUi())
      return "packet view activated";
    else 
      return "packet view deactivated";
  }

  /**
   * Show the history buffer contents.
   */
  public String command_history(String [] args) {
    int count = captureTool.dumpHistory();

    return "displayed " + count + " packets from history buffer";
  }

  /**
   * show the status.
   */
  public String command_status() {
    StringBuffer buffer = new StringBuffer();
    buffer.append("\tdevice = '" + captureTool.getDevice() + "'\n");
    buffer.append("\tsnaplen = " + snaplen + "\n");
    buffer.append("\ttimeout = " + timeout + "\n");
    buffer.append("\tfilter = '" + filter + "'\n");
    buffer.append("\tmode = '" + captureTool.getMode() + "'\n");
    try {
      // getLinkLayerType() requires an open connection
      // todo: would be better to only do this in one place, and keep track..
      captureSystem.open(captureTool.getDevice(), snaplen, 
                         promiscuous, timeout);
      buffer.append("\tlinktype = '" + 
                    captureSystem.getLinkLayerType() + "'\n");
    }
    catch(CaptureConfigurationException e) {
      // link type won't appear if device is misconfigured
    }
    catch(CaptureDeviceOpenException e) {
      // link type won't appear if device is misconfigured
    }
    buffer.append("\t" + captureSystem.getStatistics() + "\n");

    return buffer.toString();
  }

  /**
   * Capture packets.
   */
  public String command_capture(String [] args) {
    if(args.length < 1) 
      return "usage: capture packet_count";

    if(captureTool.getDevice() == null)
      return "Error. No device is specified. Try 'device' first.";

    int count = -1;
    try {
      count = Integer.parseInt(args[0]);
    }
    catch(NumberFormatException e) {
      return "Error. The capture command accepts only a numeric argument.";
    }

    try {
      System.err.println("opening device '" + captureTool.getDevice() + ".. ");
      captureSystem.open(captureTool.getDevice(), snaplen, 
                         promiscuous, timeout);
      System.err.println("setting filter '" + filter + "'.. ");
      captureSystem.setFilter(filter, true);
      System.err.println("waiting for " + count + " packet(s).. ");
      captureSystem.capture(count);
      CaptureStatistics stats = captureSystem.getStatistics();
      System.err.println(stats);
    }
    catch(CaptureDeviceOpenException cdoe) {
      System.err.println(cdoe);
      return "An error occurred. Insufficient privileges or your " + 
        "capture device isn't valid?";
    }
    catch(InvalidFilterException ife) {
      System.err.println(ife);
      return "An error occurred. Your filter is invalid?";
    }
    catch(CapturePacketException cpe) {
      System.err.println(cpe);
      return "An error occurred";
    }

    return "Capture complete.";
  }


  /**
   * Start capturing packets.
   */
  public String command_start() {

    return "";
  }

  /**
   * Generate help for commands specific to this console.
   * Invoke parent to give help on commands inherited from other consoles.
   * @param suppressed a collection of commands handled by child consoles.
   */
  protected String command_help(Collection suppressed) {
    StringBuffer buffer = new StringBuffer();
    
    // suppress help display of commands reimplemented here
    HashSet suppress = new HashSet();
    if(suppressed == null) 
      suppressed = new HashSet();

    suppress.addAll(suppressed);
    buffer.append(super.command_help(suppress));

    buffer.append("  capture:\n");
    if(! suppressed.contains("device"))
      buffer.append("\tdevice [device]         - " + 
                    "set name of the device to capture packets on\n");
    if(! suppressed.contains("snaplen"))
      buffer.append("\tsnaplen [length]        - " + 
                    "set the capture data length in bytes\n");
    if(! suppressed.contains("timeout"))
      buffer.append("\ttimeout [ms]            - " + 
                    "set the capture timeout in ms\n");
    if(! suppressed.contains("filter"))
      buffer.append("\tfilter [expression]     - " + 
                    "set the filter expression\n");
    if(! suppressed.contains("status"))
      buffer.append("\tstatus                  - " + 
                    "show current capture status\n");
    if(! suppressed.contains("capture"))
      buffer.append("\tcapture [count]         - " + 
                    "capture packets\n");
    if(! suppressed.contains("detail"))
      buffer.append("\tdetail [index]          - " + 
                    "display details on a captured packet\n");
    if(! suppressed.contains("mode"))
      buffer.append("\tmode [mode_name]        - " + 
                    "toggle capture verbosity mode\n");
    if(! suppressed.contains("history"))
      buffer.append("\thistory                 - " + 
                    "show the the capture history buffer contents\n");
    if(! suppressed.contains("clear"))
      buffer.append("\tclear                   - " + 
                    "clear the capture history buffer\n");
    if(! suppressed.contains("ui"))
      buffer.append("\tui                      - " + 
                    "toggle the packet view frame on and off\n");

    return buffer.toString();
  }


  /**
   * Tester.
   */
  public static void main(String [] args) {
    // main() is overridden because running CaptureConsole without it
    // would invoke the inherited TestConsole's main. 
    System.err.println("Use CaptureTool to invoke the console.");
  }


  /**
   * The packet capture tool associated with this console.
   */
  private CaptureTool captureTool;

  /**
   * The packet capture system interface.
   */
  private PacketCapture captureSystem;


  // todo: following should be moved into CaptureTool and reference 
  // to PacketCapture removed.

  /**
   * filter expression.
   */
  private String filter = "";

  /**
   * capture data length.
   */
  private int snaplen = PacketCapture.DEFAULT_SNAPLEN;

  /**
   * capture timeout in ms.
   */
  private int timeout = PacketCapture.DEFAULT_TIMEOUT;

  /**
   * whether or not to capture in promiscuous mode.
   */
  private boolean promiscuous = true;


  private String _rcsid = 
    "$Id: CaptureConsole.java,v 1.7 2001/05/28 21:21:22 pcharles Exp $";
}
