// $Id: IPPacket.java,v 1.6 2001/05/28 18:30:12 pcharles Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.net;

import net.ultrametrics.console.AnsiEscapeSequences;


/**
 * An IP protocol packet.
 * <p>
 * Extends an ethernet packet, adding IP header information and an IP 
 * data payload. 
 *
 * @author Patrick Charles and Jonas Lehmann
 * @version $Revision: 1.6 $
 * @lastModifiedBy $Author: pcharles $
 * @lastModifiedAt $Date: 2001/05/28 18:30:12 $
 */
public class IPPacket extends EthernetPacket
{
  /**
   * Create a new IP packet which is of a recognized protocol.
   * If the protocol code in the IP header is recognized, this 
   * constructor is used to specify the protocol.
   * For example, a recognized IP protocol is TCP.
   */
  public IPPacket(int lLen, byte [] bytes,
                  String macSourceAddress, 
                  String macDestinationAddress, 
                  String sourceAddress, 
                  String destinationAddress,
                  int protocol) {
    super(lLen, bytes, macSourceAddress, macDestinationAddress, 
          // in the link-level header, classify this packet as IP protocol
          EthernetProtocols.ETHERTYPE_IP);

    int headerLen = IPEncoding.HEADER_LEN;
    this.header = PacketEncoding.extractHeader(lLen, headerLen, bytes);
    this.data = PacketEncoding.extractData(lLen, headerLen, bytes);
    this.sourceAddress = sourceAddress;
    this.destinationAddress = destinationAddress;
    this.protocol = protocol;
  }

  /**
   * Create a new IP packet which is of an unrecognized protocol.
   * If the protocol code in the IP header is not recognized, this 
   * constructor is used.
   */
  public IPPacket(int lLen, byte [] bytes,
                  String macSourceAddress, 
                  String macDestinationAddress, 
                  String sourceAddress, 
                  String destinationAddress) {
    this(lLen, bytes, 
         macSourceAddress, macDestinationAddress, 
         sourceAddress, destinationAddress,
         IPProtocols.IPPROTO_INVALID);
  }

  /**
   * Fetch the ip header, excluding ip data.
   */
  public byte [] getHeader() {
    return header;
  }

  /**
   * Fetch data portion of the ip header.
   */
  public byte [] getData() {
    return data;
  }

  /**
   * Convert this IP packet to a readable string.
   */
  public String toString() {
    StringBuffer buffer = new StringBuffer();
    buffer.append('[');
    buffer.append(AnsiEscapeSequences.WHITE);
    buffer.append("IPPacket");
    buffer.append(AnsiEscapeSequences.RESET);
    buffer.append(": ");
    buffer.append(sourceAddress + " -> " + 
                  destinationAddress);
    buffer.append(" l=" + header.length + "," + data.length);
    buffer.append(']');

    return buffer.toString();
  }

  public String getSourceAddress() {
    return sourceAddress;
  }

  public String getDestinationAddress() {
    return destinationAddress;
  }

  public int getProtocol() {
    return protocol;
  }


  /**
   * Version. IPv4 or IPv6.
   * @see net.sourceforge.jpcap.net.IPVersion.
   */
  int version;

  /**
   * Header length.
   */
  int headerLength;

  /**
   * Type of service.
   * @see net.sourceforge.jpcap.net.TypeOfService.
   */
  int typeOfService;

  /**
   * Length.
   */
  int length;

  /**
   * Identification.
   */
  int id;

  /**
   * Fragment offset field.
   */
  int fragmentOffsetField;

  /**
   * Time to live.
   */
  int ttl;

  /**
   * Protocol.
   */
  int protocol;

  /**
   * Checksum.
   */
  int checksum;

  /**
   * Source IP Address.
   */
  String sourceAddress;

  /**
   * Destination IP Address.
   */
  String destinationAddress;

  /**
   * IP header.
   */
  byte [] header;

  /**
   * IP data.
   */
  byte [] data;

  private String _rcsid = 
    "$Id: IPPacket.java,v 1.6 2001/05/28 18:30:12 pcharles Exp $";
}
